# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: olvid/daemon/datatypes/v1/settings.proto
# Protobuf Python Version: 6.33.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    33,
    2,
    '',
    'olvid/daemon/datatypes/v1/settings.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(olvid/daemon/datatypes/v1/settings.proto\x12\x19olvid.daemon.datatypes.v1\"\xfb\x06\n\x10IdentitySettings\x12`\n\ninvitation\x18\x01 \x01(\x0b\x32@.olvid.daemon.datatypes.v1.IdentitySettings.AutoAcceptInvitationR\ninvitation\x12i\n\x11message_retention\x18\x02 \x01(\x0b\x32<.olvid.daemon.datatypes.v1.IdentitySettings.MessageRetentionR\x10messageRetention\x12P\n\x08keycloak\x18\x03 \x01(\x0b\x32\x34.olvid.daemon.datatypes.v1.IdentitySettings.KeycloakR\x08keycloak\x1a\xe6\x01\n\x14\x41utoAcceptInvitation\x12\x38\n\x18\x61uto_accept_introduction\x18\x01 \x01(\x08R\x16\x61utoAcceptIntroduction\x12*\n\x11\x61uto_accept_group\x18\x02 \x01(\x08R\x0f\x61utoAcceptGroup\x12\x32\n\x16\x61uto_accept_one_to_one\x18\x03 \x01(\x08R\x12\x61utoAcceptOneToOne\x12\x34\n\x16\x61uto_accept_invitation\x18\x04 \x01(\x08R\x14\x61utoAcceptInvitation\x1a\x9b\x02\n\x10MessageRetention\x12-\n\x12\x65xistence_duration\x18\x01 \x01(\x04R\x11\x65xistenceDuration\x12)\n\x10\x64iscussion_count\x18\x02 \x01(\x04R\x0f\x64iscussionCount\x12!\n\x0cglobal_count\x18\x03 \x01(\x04R\x0bglobalCount\x12\x38\n\x18\x63lean_locked_discussions\x18\x04 \x01(\x08R\x16\x63leanLockedDiscussions\x12P\n%preserve_is_sharing_location_messages\x18\x05 \x01(\x08R!preserveIsSharingLocationMessages\x1a\x41\n\x08Keycloak\x12\x35\n\x17\x61uto_invite_new_members\x18\x01 \x01(\x08R\x14\x61utoInviteNewMembers\"\xb6\x01\n\x12\x44iscussionSettings\x12#\n\rdiscussion_id\x18\x01 \x01(\x04R\x0c\x64iscussionId\x12\x1b\n\tread_once\x18\x02 \x01(\x08R\x08readOnce\x12-\n\x12\x65xistence_duration\x18\x03 \x01(\x04R\x11\x65xistenceDuration\x12/\n\x13visibility_duration\x18\x04 \x01(\x04R\x12visibilityDurationB\xc6\x01\n\x1d\x63om.olvid.daemon.datatypes.v1B\rSettingsProtoP\x01Z\x0folvid.io/daemon\xa2\x02\x03ODD\xaa\x02\x19Olvid.Daemon.Datatypes.V1\xca\x02\x19Olvid\\Daemon\\Datatypes\\V1\xe2\x02%Olvid\\Daemon\\Datatypes\\V1\\GPBMetadata\xea\x02\x1cOlvid::Daemon::Datatypes::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'olvid.daemon.datatypes.v1.settings_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\035com.olvid.daemon.datatypes.v1B\rSettingsProtoP\001Z\017olvid.io/daemon\242\002\003ODD\252\002\031Olvid.Daemon.Datatypes.V1\312\002\031Olvid\\Daemon\\Datatypes\\V1\342\002%Olvid\\Daemon\\Datatypes\\V1\\GPBMetadata\352\002\034Olvid::Daemon::Datatypes::V1'
  _globals['_IDENTITYSETTINGS']._serialized_start=72
  _globals['_IDENTITYSETTINGS']._serialized_end=963
  _globals['_IDENTITYSETTINGS_AUTOACCEPTINVITATION']._serialized_start=380
  _globals['_IDENTITYSETTINGS_AUTOACCEPTINVITATION']._serialized_end=610
  _globals['_IDENTITYSETTINGS_MESSAGERETENTION']._serialized_start=613
  _globals['_IDENTITYSETTINGS_MESSAGERETENTION']._serialized_end=896
  _globals['_IDENTITYSETTINGS_KEYCLOAK']._serialized_start=898
  _globals['_IDENTITYSETTINGS_KEYCLOAK']._serialized_end=963
  _globals['_DISCUSSIONSETTINGS']._serialized_start=966
  _globals['_DISCUSSIONSETTINGS']._serialized_end=1148
# @@protoc_insertion_point(module_scope)



import asyncio
import base64
import os
from typing import Callable, Optional, Coroutine, AsyncIterator, TYPE_CHECKING

if TYPE_CHECKING:
	from ..core.OlvidClient import OlvidClient
	from ..internal.types import NotificationMessageType

from ..core.logger import core_logger
from .Notifications import NOTIFICATIONS


class GenericNotificationListener:
	"""
	GenericNotificationListener a basic notification listener to use within OlvidClient and other subclasses.

	A listener is a method called every time a notification is triggered. When you add a listener to an OlvidClient it will
	automatically register to this notification and handler will be called every time a notification is sent by daemon.

	We do not recommend that you use GenericNotificationListener directly. Instead, you should use one of the provided
	listeners in the ListenersImplementation file.
	You can access them like this:
	```
	from olvid import listeners
	listeners.MessageReceivedListener(handler=lambda m: a)
	```
	Like this you won't need to specify the notification you want to listen to.
	Also, you won't need to use protobuf Notification messages, message are already un-wrapped and handler ill receive notification content.
	For example MessageReceivedListener.handler will receive a datatypes.Message item, not a MessageReceivedNotification
	as a GenericNotificationListener will receive if listening to MessageReceivedNotification.
	"""
	def __init__(self, notification_type: NOTIFICATIONS, handler: Callable[[type["NotificationMessageType"]], Optional[Coroutine]], iterator_args: dict = None):
		self._notification_type: NOTIFICATIONS = notification_type
		self._handler = handler

		self._iterator_args: dict = iterator_args if iterator_args else {}

		# we compute listener key after end of init to let child classes fill iterator args
		self._listener_key: Optional[str] = None

	@property
	def notification_type(self) -> NOTIFICATIONS:
		return self._notification_type

	@property
	def listener_key(self) -> str:
		if self._listener_key is None:
			self._listener_key = self._generate_listener_key()
		return self._listener_key

	async def handle_notification(self, notification_message: type["NotificationMessageType"]):
		try:
			res = self._handler(notification_message)
			if asyncio.iscoroutine(res):
				await res
		except Exception:
			core_logger.exception(f"{self.__class__.__name__}: unexpected exception: {self}")

	def _create_iterator(self, client: "OlvidClient") -> AsyncIterator:
		return getattr(client, f"_notif_{self.notification_type.name.lower()}")(**self._iterator_args)

	def _generate_listener_key(self) -> str:
		key: str = f"{self.notification_type.name}"
		# if listener have iterator args we create a unique key (daemon response stream might differ even with same parameters, for example if a count parameter is set)
		if len(self._iterator_args) >= 1:
			for k, v in self._iterator_args.items():
				key += f"_{k}-{v}"
			key += f"_{base64.b64encode(os.urandom(12))}"
		return key

	def __str__(self):
		return f"{self.listener_key}: {self._handler.__name__} (arguments: {self._iterator_args})"

	def __repr__(self):
		return self.__str__()



from .logger import tools_logger
from .KeycloakAdminApiClient import KeycloakAdminApiClient
from .MonitoringBot import MonitoringBot
from .DelayedTasksHolder import DelayedTasksHolder
from .ChatBot import ChatBot

del logger, tools_logger
